@@ -5,18 +5,35 @@
 #include "my-object.h"
 #include "my-object-marshal.h"
 
+static gboolean my_object_throw_error_under_score (MyObject *obj,
+    GError **error);
+
 #include "test-service-glib-glue.h"
 
+void
+my_object_register_marshallers (void)
+{
+  dbus_g_object_register_marshaller (my_object_marshal_VOID__STRING_INT_STRING,
+      G_TYPE_NONE, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INVALID);
+
+  dbus_g_object_register_marshaller (my_object_marshal_VOID__STRING_BOXED,
+      G_TYPE_NONE, G_TYPE_STRING, G_TYPE_VALUE, G_TYPE_INVALID);
+}
+
 /* Properties */
 enum
 {
   PROP_0,
-  PROP_THIS_IS_A_STRING
+  PROP_THIS_IS_A_STRING,
+  PROP_NO_TOUCHING,
+  PROP_SUPER_STUDLY,
+  PROP_SHOULD_BE_HIDDEN
 };
 
 enum
 {
   FROBNICATE,
+  OBJECTIFIED,
   SIG0,
   SIG1,
   SIG2,
@@ -53,7 +70,19 @@ my_object_set_property (GObject      *object,
       g_free (mobject->this_is_a_string);
       mobject->this_is_a_string = g_value_dup_string (value);
       break;
-      
+
+    case PROP_NO_TOUCHING:
+      mobject->notouching = g_value_get_uint (value);
+      break;
+
+    case PROP_SUPER_STUDLY:
+      mobject->super_studly = g_value_get_double (value);
+      break;
+
+    case PROP_SHOULD_BE_HIDDEN:
+      mobject->should_be_hidden = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -75,7 +104,19 @@ my_object_get_property (GObject      *object,
     case PROP_THIS_IS_A_STRING:
       g_value_set_string (value, mobject->this_is_a_string);
       break;
-      
+
+    case PROP_NO_TOUCHING:
+      g_value_set_uint (value, mobject->notouching);
+      break;
+
+    case PROP_SUPER_STUDLY:
+      g_value_set_double (value, mobject->super_studly);
+      break;
+
+    case PROP_SHOULD_BE_HIDDEN:
+      g_value_set_boolean (value, mobject->should_be_hidden);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -86,6 +127,7 @@ static void
 my_object_init (MyObject *obj)
 {
   obj->val = 0;
+  obj->notouching = 42;
 }
 
 static void
@@ -93,6 +135,8 @@ my_object_class_init (MyObjectClass *mobject_class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (mobject_class);
 
+  my_object_register_marshallers ();
+
   dbus_g_object_type_install_info (MY_TYPE_OBJECT,
 				   &dbus_glib_my_object_object_info);
 
@@ -107,6 +151,30 @@ my_object_class_init (MyObjectClass *mobject_class)
                                                         _("Example of a string property"),
                                                         "default value",
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_NO_TOUCHING,
+                                   g_param_spec_uint ("no_touching",
+                                                      _("Don't touch"),
+                                                      _("Example of a readonly property (for export)"),
+                                                      0, 100, 42,
+                                                      G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                				   PROP_SUPER_STUDLY,
+				                   g_param_spec_double ("super-studly",
+                                                        _("In Studly Caps"),
+                                                        _("Example of a StudlyCaps property"),
+                                                        0, 256, 128,
+                                                        G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                				   PROP_SHOULD_BE_HIDDEN,
+				                   g_param_spec_boolean ("should-be-hidden",
+                                                        _("A non-exported property"),
+                                                        _("Example of a property we don't want exported"),
+                                                        FALSE,
+                                                        G_PARAM_READWRITE));
+
   signals[FROBNICATE] =
     g_signal_new ("frobnicate",
 		  G_OBJECT_CLASS_TYPE (mobject_class),
@@ -116,6 +184,15 @@ my_object_class_init (MyObjectClass *mobject_class)
                   g_cclosure_marshal_VOID__INT,
                   G_TYPE_NONE, 1, G_TYPE_INT);
 
+  signals[OBJECTIFIED] =
+    g_signal_new ("objectified",
+                  G_OBJECT_CLASS_TYPE (mobject_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, G_TYPE_OBJECT);
+
   signals[SIG0] =
     g_signal_new ("sig0",
 		  G_OBJECT_CLASS_TYPE (mobject_class),
@@ -170,6 +247,7 @@ my_object_error_get_type (void)
 			ENUM_ENTRY (MY_OBJECT_ERROR_FOO, "Foo"),
 			ENUM_ENTRY (MY_OBJECT_ERROR_BAR, "Bar"),
 			ENUM_ENTRY (MY_OBJECT_ERROR_MULTI_WORD, "Multi-word"),
+			ENUM_ENTRY (MY_OBJECT_ERROR_UNDER_SCORE, "Under_score"),
 			{ 0, 0, 0 }
 		};
 
@@ -246,6 +324,29 @@ my_object_throw_error_multi_word (MyObject *obj, GError **error)
   return FALSE;
 }
 
+static gboolean
+my_object_throw_error_under_score (MyObject *obj, GError **error)
+{
+  g_set_error (error,
+	       MY_OBJECT_ERROR,
+	       MY_OBJECT_ERROR_UNDER_SCORE,
+	       "%s",
+	       "this method's error has an underscore");
+  return FALSE;
+}
+
+gboolean
+my_object_throw_unregistered_error (MyObject *obj, GError **error)
+{
+  /* Unregistered errors shouldn't cause a dbus abort.  See
+   * https://bugzilla.redhat.com/show_bug.cgi?id=581794
+   */
+  g_set_error (error, 0, 0,
+	       "%s",
+	       "this method always loses more");
+  return FALSE;
+}
+
 
 gboolean
 my_object_uppercase (MyObject *obj, const char *str, char **ret, GError **error)
@@ -773,6 +874,11 @@ my_object_async_throw_error (MyObject *obj, DBusGMethodInvocation *context)
   g_idle_add ((GSourceFunc)do_async_error,  data);
 }
 
+void
+my_object_unsafe_disable_legacy_property_access (MyObject *obj)
+{
+  dbus_glib_global_set_disable_legacy_property_access ();
+}
 
 extern GMainLoop *loop;
 
@@ -782,3 +888,10 @@ my_object_terminate (MyObject *obj, GError **error)
   g_main_loop_quit (loop);
   return TRUE;
 }
+
+void
+my_object_emit_objectified (MyObject *obj,
+    GObject *other)
+{
+  g_signal_emit (obj, signals[OBJECTIFIED], 0, other);
+}
