@@ -1,3 +1,5 @@
+#include <config.h>
+
 /* -*- mode: C; c-file-style: "gnu" -*- */
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
@@ -8,9 +10,10 @@
 #include <dbus/dbus-gparser.h>
 #include <glib.h>
 #include <glib-object.h>
-#include "my-object-marshal.h"
+#include "my-object.h"
+
+GMainLoop *loop = NULL;
 
-static GMainLoop *loop = NULL;
 static const char *await_terminating_service = NULL;
 static int n_times_foo_received = 0;
 static int n_times_frobnicate_received = 0;
@@ -27,7 +30,8 @@ static gboolean proxy_destroy_and_nameowner_complete = FALSE;
 static DBusGProxy *test_terminate_proxy1 = NULL;
 static DBusGProxy *test_terminate_proxy2 = NULL;
 
-static void lose (const char *fmt, ...) G_GNUC_NORETURN G_GNUC_PRINTF (1, 2);
+#define lose(...) g_error (__VA_ARGS__)
+
 static void lose_gerror (const char *prefix, GError *error) G_GNUC_NORETURN;
 
 static void
@@ -463,21 +467,6 @@ test_subclass_get_all (DBusGConnection *connection,
   return TRUE;
 }
 
-static void
-lose (const char *str, ...)
-{
-  va_list args;
-
-  va_start (args, str);
-
-  vfprintf (stderr, str, args);
-  fputc ('\n', stderr);
-
-  va_end (args);
-
-  exit (1);
-}
-
 static void
 lose_gerror (const char *prefix, GError *error) 
 {
@@ -485,7 +474,8 @@ lose_gerror (const char *prefix, GError *error)
     lose ("%s (%s): %s", prefix, dbus_g_error_get_name (error),
 	  error->message);
   else
-    lose ("%s: %s", prefix, error->message);
+    lose ("%s: %s#%d: %s", prefix, g_quark_to_string (error->domain),
+        error->code, error->message);
 }
 
 static void
@@ -753,6 +743,9 @@ main (int argc, char **argv)
 				  increment_received_cb, g_strdup ("moo"), g_free,
 				  G_TYPE_UINT, 42,
 				  G_TYPE_INVALID);
+  if (call == NULL)
+    lose ("Failed to begin Increment call");
+
   dbus_g_connection_flush (connection);
   exit_timeout = g_timeout_add (5000, timed_exit, loop);
   g_main_loop_run (loop);
@@ -896,6 +889,17 @@ main (int argc, char **argv)
   g_print ("(wrapped) ThrowErrorMultiWord failed (as expected) returned error: %s\n", error->message);
   g_clear_error (&error);
 
+  g_print ("Calling (wrapped) throw_error_under_score\n");
+  if (org_freedesktop_DBus_GLib_Tests_MyObject_throw_error_under_score (proxy, &error) != FALSE)
+    lose ("(wrapped) ThrowErrorUnderScore call unexpectedly succeeded!");
+
+  g_assert_error (error, DBUS_GERROR, DBUS_GERROR_REMOTE_EXCEPTION);
+  g_assert_cmpstr (dbus_g_error_get_name (error), ==,
+      "org.freedesktop.DBus.GLib.Tests.MyObject.Under_score");
+
+  g_print ("(wrapped) ThrowErrorUnderScore failed (as expected) returned error: %s\n", error->message);
+  g_clear_error (&error);
+
   if (org_freedesktop_DBus_GLib_Tests_MyObject_async_throw_error (proxy, &error) != FALSE)
     lose ("(wrapped) AsyncThrowError call unexpectedly succeeded!");
 
@@ -1703,11 +1707,7 @@ main (int argc, char **argv)
   if (proxy == NULL)
     lose_gerror ("Failed to create proxy for name owner", error);
 
-  dbus_g_object_register_marshaller (my_object_marshal_VOID__STRING_INT_STRING, 
-				     G_TYPE_NONE, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INVALID);
-
-  dbus_g_object_register_marshaller (my_object_marshal_VOID__STRING_BOXED, 
-				     G_TYPE_NONE, G_TYPE_STRING, G_TYPE_VALUE, G_TYPE_INVALID);
+  my_object_register_marshallers ();
 
   dbus_g_proxy_add_signal (proxy, "Sig0", G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INVALID);
   dbus_g_proxy_add_signal (proxy, "Sig1", G_TYPE_STRING, G_TYPE_VALUE, G_TYPE_INVALID);
